/*
  This file is declared to be in the Public Domain by its author,
  Ronald S. Burkey

  Filename:     modernHAL-S-FC.c
  Purpose:      A compiler front-end for the HAL/S language.  Generally not
                for direct usage, but instead via yaHAL-S-FC.py.
  Compiler:     GNU gcc.
  Reference:    http://www.ibibio.org/apollo
  Mods:         2022-12-12 RSB  Adapted from a makefile auto-generated by
                                BNF Converter.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Parser.h"
#include "Printer.h"
#include "Absyn.h"

void usage(void) {
  printf("Usage:\n");
  printf("\tmodernHAL-S-FC [OPTIONS] SOURCEFILE | <SOURCEFILE\n\n");
  printf("The available OPTIONS are:\n");
  printf("\t--help            Print this message and quit.\n");
  printf("\t--trace           Enable parser tracing.\n");
}

int main(int argc, char ** argv)
{
  FILE *input;
  COMPILATION parse_tree;
  int quiet = 0;
  char *filename = NULL;
  int i;

  input = stdin;
  for (i = 1; i < argc; i++)
    {
      if (!strcmp(argv[i], "--help"))
        {
          usage();
          exit(0);
        }
      else if (!strcmp(argv[i], "--trace"))
        {
          extern int HAL_Sdebug;
          HAL_Sdebug = 1;
        }
      else if (argv[i][0] == '-')
        {
          printf("Unrecognized option %s.\n\n", argv[i]);
          usage();
          exit(1);
        }
      else if (filename != NULL)
        {
          printf("Multiple filenames given (%s, %s)\n\n", filename, argv[i]);
          usage();
          exit(1);
        }
      else
        {
          filename = argv[i];
          input = fopen(filename, "r");
          if (input == NULL)
            {
              printf("Source file not found (%s)\n\n", filename);
              usage();
              exit(1);
            }
        }
    }

  /* The default entry point is used. For other options see Parser.h */
  parse_tree = pCOMPILATION(input);
  if (parse_tree)
  {
    if (!quiet) {
      printf("%s\n", showCOMPILATION(parse_tree));
    }
    return 0;
  }
  return 1;
}

